<?php

declare(strict_types=1);

namespace Drupal\Tests\data_pipelines_elasticsearch\Kernel;

use Drupal\data_pipelines\DatasetData;
use Drupal\data_pipelines\DatasetPipelinePluginManager;
use Drupal\data_pipelines\Entity\DestinationInterface;
use Drupal\data_pipelines_elasticsearch\Exception\DatasetNotRemovedFromIndexException;
use Drupal\data_pipelines_elasticsearch\Plugin\DatasetDestination\ElasticSearchDestination;
use Drupal\Tests\data_pipelines\Kernel\DatasetKernelTestBase;
use Elasticsearch\Client;
use Elasticsearch\Namespaces\IndicesNamespace;
use Prophecy\Argument;
use Psr\Log\Test\TestLogger;

/**
 * Defines a class for testing elastic indexer.
 *
 * @group data_pipelines
 *
 * @covers \Drupal\data_pipelines_elasticsearch\Plugin\DatasetDestination\ElasticSearchDestination
 * @covers \Drupal\data_pipelines_elasticsearch\Exception\DatasetNotRemovedFromIndexException
 */
class ElasticsearchDestinationTest extends DatasetKernelTestBase {

  /**
   * Tests index creation.
   */
  public function testIndexCreation() {
    $dataset = $this->createTestDataset();
    $indices = $this->prophesize(IndicesNamespace::class);
    $indices->exists(Argument::any())->willReturn(FALSE);
    $destination = $this->prophesize(DestinationInterface::class);

    $indices->create([
      'index' => 'foo_' . $dataset->getMachineName(),
      'body' => [
        'settings' => [
          'number_of_shards' => ElasticSearchDestination::INDEX_SHARDS,
          'number_of_replicas' => ElasticSearchDestination::INDEX_REPLICAS,
          'codec' => ElasticSearchDestination::INDEX_CODEC,
        ],
      ],
    ])->shouldBeCalled();
    $client = $this->prophesize(Client::class);
    $client->indices()->willReturn($indices->reveal());

    $client->index(Argument::any())->willReturn([]);

    $pipeline_manager = $this->prophesize(DatasetPipelinePluginManager::class);
    $pipeline_manager->hasDefinition(Argument::exact($dataset->getPipelineId()))->willReturn(TRUE);
    $pipeline_manager->getDefinition(Argument::exact($dataset->getPipelineId()))->willReturn([]);

    $configuration = [
      'prefix' => 'foo_',
    ];
    $plugin_id = 'foo';
    $plugin_definition = [];
    $logger = new TestLogger();
    $indexer = new ElasticSearchDestination($configuration, $plugin_id, $plugin_definition, $pipeline_manager->reveal(), $logger, $client->reveal());
    $this->assertTrue($indexer->saveDataSet($dataset, $destination->reveal()));
  }

  /**
   * Tests index not created if exists.
   */
  public function testIndexNotCreatedIfExists() {
    $dataset = $this->createTestDataset();
    $indices = $this->prophesize(IndicesNamespace::class);
    $indices->exists(Argument::any())->willReturn(TRUE);
    $indices->create(Argument::any())->shouldNotBeCalled();
    $client = $this->prophesize(Client::class);
    $client->indices()->willReturn($indices->reveal());

    $client->index(Argument::any())->willReturn([]);

    $pipeline_manager = $this->prophesize(DatasetPipelinePluginManager::class);
    $pipeline_manager->hasDefinition(Argument::exact($dataset->getPipelineId()))->willReturn(TRUE);
    $pipeline_manager->getDefinition(Argument::exact($dataset->getPipelineId()))->willReturn([]);

    $configuration = [
      'prefix' => 'foo_',
    ];
    $plugin_id = 'foo';
    $plugin_definition = [];
    $logger = new TestLogger();
    $indexer = new ElasticSearchDestination($configuration, $plugin_id, $plugin_definition, $pipeline_manager->reveal(), $logger, $client->reveal());
    $destination = $this->prophesize(DestinationInterface::class);
    $this->assertTrue($indexer->saveDataSet($dataset, $destination->reveal()));
  }

  /**
   * Tests index existence failure.
   */
  public function testIndexExistenceFailure() {
    $dataset = $this->createTestDataset();
    $indices = $this->prophesize(IndicesNamespace::class);
    $indices->exists(Argument::any())->willThrow(new \Exception('Barf'));
    $indices->create(Argument::any())->shouldNotBeCalled();
    $client = $this->prophesize(Client::class);
    $client->indices()->willReturn($indices->reveal());

    $pipeline_manager = $this->prophesize(DatasetPipelinePluginManager::class);
    $pipeline_manager->hasDefinition(Argument::exact($dataset->getPipelineId()))->willReturn(TRUE);
    $pipeline_manager->getDefinition(Argument::exact($dataset->getPipelineId()))->willReturn([]);

    $configuration = [
      'prefix' => 'foo_',
    ];
    $plugin_id = 'foo';
    $plugin_definition = [];
    $logger = new TestLogger();
    $indexer = new ElasticSearchDestination($configuration, $plugin_id, $plugin_definition, $pipeline_manager->reveal(), $logger, $client->reveal());

    $destination = $this->prophesize(DestinationInterface::class);
    $this->assertFalse($indexer->saveDataSet($dataset, $destination->reveal()));
  }

  /**
   * Tests index creation failure.
   */
  public function testIndexCreationFailure() {
    $dataset = $this->createTestDataset();
    $indices = $this->prophesize(IndicesNamespace::class);
    $indices->exists(Argument::any())->willReturn(FALSE);
    $indices->create(Argument::any())->willThrow(new \Exception('Barf'));
    $client = $this->prophesize(Client::class);
    $client->indices()->willReturn($indices->reveal());

    $pipeline_manager = $this->prophesize(DatasetPipelinePluginManager::class);
    $pipeline_manager->hasDefinition(Argument::exact($dataset->getPipelineId()))->willReturn(TRUE);
    $pipeline_manager->getDefinition(Argument::exact($dataset->getPipelineId()))->willReturn([]);

    $configuration = [
      'prefix' => 'foo_',
    ];
    $plugin_id = 'foo';
    $plugin_definition = [];
    $logger = new TestLogger();
    $indexer = new ElasticSearchDestination($configuration, $plugin_id, $plugin_definition, $pipeline_manager->reveal(), $logger, $client->reveal());

    $dataset = $this->createTestDataset();
    $destination = $this->prophesize(DestinationInterface::class);
    $this->assertFalse($indexer->saveDataSet($dataset, $destination->reveal()));
  }

  /**
   * Tests indexing.
   */
  public function testIndexing() {
    $dataset = $this->createTestDataset();
    $indices = $this->prophesize(IndicesNamespace::class);
    $destination = $this->prophesize(DestinationInterface::class);
    $indices->exists(Argument::any())->willReturn(TRUE);
    $client = $this->prophesize(Client::class);
    $client->indices()->willReturn($indices->reveal());

    $pipeline_manager = $this->prophesize(DatasetPipelinePluginManager::class);
    $pipeline_manager->hasDefinition(Argument::exact($dataset->getPipelineId()))->willReturn(TRUE);
    $pipeline_manager->getDefinition(Argument::exact($dataset->getPipelineId()))->willReturn([]);

    foreach ($dataset->data() as $delta => $data) {
      assert($data instanceof DatasetData);
      $expected = [
        'index' => 'foo_' . $dataset->getMachineName(),
        'id' => $dataset->getMachineName() . ':' . $delta,
        'body' => $data->getArrayCopy(),
      ];
      $client->index($expected)->shouldBeCalled();
    }

    $configuration = [
      'prefix' => 'foo_',
    ];
    $plugin_id = 'foo';
    $plugin_definition = [];
    $logger = new TestLogger();
    $indexer = new ElasticSearchDestination($configuration, $plugin_id, $plugin_definition, $pipeline_manager->reveal(), $logger, $client->reveal());
    $this->assertTrue($indexer->saveDataSet($dataset, $destination->reveal()));
  }

  /**
   * Tests deletion.
   */
  public function testDeleting() {
    $dataset = $this->createTestDataset();
    $destination = $this->prophesize(DestinationInterface::class);
    $indices = $this->prophesize(IndicesNamespace::class);
    $client = $this->prophesize(Client::class);
    $client->indices()->willReturn($indices->reveal());

    $pipeline_manager = $this->prophesize(DatasetPipelinePluginManager::class);
    $pipeline_manager->hasDefinition(Argument::exact($dataset->getPipelineId()))->willReturn(TRUE);
    $pipeline_manager->getDefinition(Argument::exact($dataset->getPipelineId()))->willReturn([]);

    $expected = [
      'index' => 'foo_' . $dataset->getMachineName(),
    ];
    $indices->delete($expected)->shouldBeCalled();

    $configuration = [
      'prefix' => 'foo_',
    ];
    $plugin_id = 'foo';
    $plugin_definition = [];
    $logger = new TestLogger();
    $indexer = new ElasticSearchDestination($configuration, $plugin_id, $plugin_definition, $pipeline_manager->reveal(), $logger, $client->reveal());
    $this->assertTrue($indexer->deleteDataSet($dataset, $destination->reveal()));
  }

  /**
   * Tests deletion failure.
   */
  public function testDeletingException() {
    $dataset = $this->createTestDataset();
    $destination = $this->prophesize(DestinationInterface::class);
    $indices = $this->prophesize(IndicesNamespace::class);
    $client = $this->prophesize(Client::class);
    $client->indices()->willReturn($indices->reveal());

    $pipeline_manager = $this->prophesize(DatasetPipelinePluginManager::class);
    $pipeline_manager->hasDefinition(Argument::exact($dataset->getPipelineId()))->willReturn(TRUE);
    $pipeline_manager->getDefinition(Argument::exact($dataset->getPipelineId()))->willReturn([]);

    $expected = [
      'index' => 'foo_' . $dataset->getMachineName(),
    ];
    $indices->delete($expected)->willThrow(new \Exception('barf'));

    $configuration = [
      'prefix' => 'foo_',
    ];
    $plugin_id = 'foo';
    $plugin_definition = [];
    $logger = new TestLogger();
    $indexer = new ElasticSearchDestination($configuration, $plugin_id, $plugin_definition, $pipeline_manager->reveal(), $logger, $client->reveal());

    $this->expectException(DatasetNotRemovedFromIndexException::class);
    $this->expectExceptionMessage(sprintf('Could not remove dataset %s from the index, error was barf', $dataset->label()));
    $indexer->deleteDataSet($dataset, $destination->reveal());
  }

}
