<?php

namespace Drupal\data_pipelines_test\Plugin\DatasetDestination;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\data_pipelines\Destination\DatasetDestinationPluginBase;
use Drupal\data_pipelines\Entity\DatasetInterface;
use Drupal\data_pipelines\Entity\DestinationInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * A class for providing JSON as an output.
 *
 * @DatasetDestination(
 *   id="state",
 *   label="State",
 *   description="A state destination used for testing"
 * )
 */
class StateDestination extends DatasetDestinationPluginBase implements ContainerFactoryPluginInterface {

  /**
   * The state.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * Constructs a new FileDestinationBase.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\State\StateInterface $state
   *   The state service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, StateInterface $state) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->state = $state;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('state')
    );
  }

  /**
   * Gets the state key.
   *
   * @return string
   *   The state key.
   */
  public function getStateKey(): string {
    return $this->configuration['state_key'] ?? 'data_pipelines_test';
  }

  /**
   * Gets the state data.
   *
   * @return array
   *   The data.
   */
  public function getData(): array {
    return $this->state->get($this->getStateKey()) ?? [];
  }

  /**
   * Sets the state data.
   *
   * @param array $data
   *   The data.
   */
  public function setData(array $data): void {
    $this->state->set($this->getStateKey(), $data);
  }

  /**
   * {@inheritdoc}
   */
  public function saveDataSet(DatasetInterface $dataset, DestinationInterface $destination): bool {
    $data = $this->getData();
    $data[$destination->id()] = iterator_to_array($dataset->data());
    $this->setData($data);
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function deleteDataSet(DatasetInterface $dataset, DestinationInterface $destination): bool {
    $data = $this->getData();
    unset($data[$destination->id()]);
    $this->setData($data);
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
  }

  /**
   * {@inheritdoc}
   */
  public function viewSettings(): array {
    return [
      'data' => [
        '#markup' => new TranslatableMarkup("Only for testing"),
      ],
    ];
  }

}
