<?php

declare(strict_types=1);

namespace Drupal\Tests\data_pipelines\Kernel;

use Drupal\Core\Queue\QueueWorkerManagerInterface;
use Drupal\data_pipelines\Destination\DestinationSaveRequest;
use Drupal\data_pipelines\Entity\DatasetInterface;
use Drupal\data_pipelines\Form\DatasetBatchOperations;
use Drupal\data_pipelines\Plugin\QueueWorker\DestinationWorker;

/**
 * Defines a class for testing queue indexing.
 *
 * @group data_pipelines
 *
 * @covers \Drupal\data_pipelines\EntityHandlers\DatasetStorage
 * @covers \Drupal\data_pipelines\Plugin\QueueWorker\DestinationWorker
 */
class DestinationQueueTest extends DatasetKernelTestBase {

  /**
   * Tests queue derivation.
   *
   * @covers \Drupal\data_pipelines\Plugin\Derivative\DatasetQueueWorkerDeriver
   */
  public function testQueueDerivation() {
    $queue = \Drupal::service('plugin.manager.queue_worker');
    assert($queue instanceof QueueWorkerManagerInterface);
    $this->assertCount(0, array_filter(array_keys($queue->getDefinitions()), function (string $id) {
      return strpos($id, 'data_pipelines_process') === 0;
    }));
    $dataset = $this->createTestDataset();
    $this->assertCount(1, array_filter(array_keys($queue->getDefinitions()), function (string $id) {
      return strpos($id, 'data_pipelines_process') === 0;
    }));
    $dataset->delete();
    $this->assertCount(0, array_filter(array_keys($queue->getDefinitions()), function (string $id) {
      return strpos($id, 'data_pipelines_process') === 0;
    }));
  }

  /**
   * Tests save and deletion.
   */
  public function testSaveAndDeletion() {
    $dataset = $this->createTestDataset();
    batch_set(DatasetBatchOperations::batchForDataset($dataset));
    $queue = \Drupal::queue(DatasetInterface::QUEUE_NAME_PREFIX . $dataset->id());
    $this->assertEquals(1, $queue->numberOfItems());
    $item = $queue->claimItem();
    $this->assertEquals(new DestinationSaveRequest($dataset->getMachineName(), $dataset->label()), $item->data);
    $new_label = $this->randomMachineName();
    $dataset->name = $new_label;
    $dataset->save();
    // Should still be one item, as the old one is now invalid and has been
    // deleted.
    $this->assertEquals(1, $queue->numberOfItems());
    $item = $queue->claimItem();
    $this->assertEquals(new DestinationSaveRequest($dataset->getMachineName(), $dataset->label()), $item->data);

    // Assert saving while syncing does not queue.
    $queue->deleteQueue();
    $this->assertEquals(0, $queue->numberOfItems());
    $dataset->setSyncing(TRUE);
    $dataset->save();
    $this->assertEquals(0, $queue->numberOfItems());
    $dataset->setSyncing(FALSE);
    $dataset->save();
    $this->assertEquals(1, $queue->numberOfItems());

    $dataset->delete();
    // Should still be one item, as the old one is now invalid and has been
    // deleted.
    $this->assertEquals(1, $queue->numberOfItems());
    $item = $queue->claimItem();
    $this->assertEquals(new DestinationSaveRequest($dataset->getMachineName(), $dataset->label()), $item->data);
  }

  /**
   * Tests indexing.
   */
  public function testQueueWorker() {
    $destinationId = $this->randomMachineName();
    $dataset = $this->createTestDataset(['destinationId' => $destinationId]);
    $queue_id = DataSetInterface::QUEUE_NAME_PREFIX . $dataset->id();
    $queue_manager = \Drupal::service('plugin.manager.queue_worker');
    $queue = \Drupal::queue($queue_id);
    $this->assertEquals(1, $queue->numberOfItems());
    assert($queue_manager instanceof QueueWorkerManagerInterface);
    $this->assertTrue($queue_manager->hasDefinition($queue_id));
    $instance = $queue_manager->createInstance($queue_id);
    assert($instance instanceof DestinationWorker);
    $instance->processItem($queue->claimItem()->data);

    $data = \Drupal::state()->get('data_pipelines_test');
    $this->assertEquals([$destinationId => iterator_to_array($dataset->data())], $data);
    $this->assertFalse($queue->claimItem());

    $expected = $data[$destinationId][0];

    $this->assertTrue($expected['should_we']);
    $this->assertEquals('bloggs, joe', $expected['full_name']);

    // Clean up.
    $dataset->delete();
    $data = \Drupal::state()->get('data_pipelines_test');
    $this->assertEmpty($data);

  }

}
