<?php

declare(strict_types=1);

namespace Drupal\Tests\data_pipelines\Kernel\Form;

use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\data_pipelines\Entity\DatasetInterface;
use Drupal\data_pipelines\Form\DatasetBatchOperations;
use Drupal\Tests\data_pipelines\Kernel\DatasetKernelTestBase;

/**
 * Defines a class for testing DatasetBatchOperations.
 *
 * @group data_pipelines
 *
 * @covers \Drupal\data_pipelines\Form\DatasetBatchOperations
 */
class DatasetBatchOperationsTest extends DatasetKernelTestBase {

  /**
   * Tests batch builder.
   */
  public function testBatchForDataset(): void {
    $dataset = $this->createTestDataset();
    $batch = DatasetBatchOperations::batchForDataset($dataset);
    $this->assertEquals([
      [[DatasetBatchOperations::class, 'operationValidate'], [$dataset->id()]],
      [[DatasetBatchOperations::class, 'operationQueueItem'], [$dataset->id()]],
      [[DatasetBatchOperations::class, 'operationProcess'], [$dataset->id()]],
    ], $batch['operations']);
  }

  /**
   * Tests validation operation.
   */
  public function testValidation(): void {
    $dataset = $this->createTestDataset();
    $context = [
      'results' => [],
    ];
    DatasetBatchOperations::operationValidate((int) $dataset->id(), $context);
    $this->assertEquals('Validation complete.', (string) $context['message']);
    $csv = <<<CSV
should_we,firstname,lastname
Y,jo,bloggs
N,betty,bloggs
CSV;
    $dataset = $this->createTestDataset([
      'csv_text' => $csv,
    ]);
    DatasetBatchOperations::operationValidate((int) $dataset->id(), $context);
    $dataset = $this->reloadDataset($dataset);
    $this->assertEquals('Validation complete, validation errors exist for dataset (' . $dataset->label() . ').', (string) $context['message']);
    $this->assertEquals([
      'Firstname should be at least 3 characters',
    ], $dataset->getLogs());
    $this->assertTrue($dataset->isPendingValidation());
  }

  /**
   * Tests process operation with failed validation.
   */
  public function testProcessingWhenInvalid(): void {
    $dataset = $this->createTestDataset();
    $context = [
      'results' => [],
    ];
    DatasetBatchOperations::operationProcess((int) $dataset->id(), $context);
    $this->assertEquals('Skipped processing dataset, validation errors exist.', (string) $context['message']);
  }

  /**
   * Tests queue operation.
   */
  public function testQueuingValid(): void {
    $dataset = $this->createTestDataset([
      'status' => DatasetInterface::STATUS_PENDING_PROCESSING,
    ]);
    $queue = \Drupal::queue(DataSetInterface::QUEUE_NAME_PREFIX . $dataset->id());
    // Clear any previous operations.
    $queue->deleteQueue();
    $context = [
      'results' => [],
    ];
    DatasetBatchOperations::operationQueueItem((int) $dataset->id(), $context);
    $this->assertEquals('Data is successfully queued for indexing.', (string) $context['message']);
    $this->assertEquals(1, $queue->numberOfItems());
  }

  /**
   * Tests process operation.
   */
  public function testProcessingValid(): void {
    $dataset = $this->createTestDataset([
      'status' => DatasetInterface::STATUS_PENDING_PROCESSING,
    ]);
    $queue = \Drupal::queue(DataSetInterface::QUEUE_NAME_PREFIX . $dataset->id());
    $this->assertEquals(1, $queue->numberOfItems());
    $context = [
      'results' => [],
    ];
    DatasetBatchOperations::operationProcess((int) $dataset->id(), $context);
    $this->assertEquals('Processed dataset.', (string) $context['message']);
    $this->assertEquals(0, $queue->numberOfItems());
  }

  /**
   * Tests finished with success.
   */
  public function testFinishedSuccess(): void {
    $dataset = $this->createTestDataset([
      'status' => DatasetInterface::STATUS_PENDING_PROCESSING,
    ]);
    $context = [
      'results' => [
        'dataset_id' => $dataset->id(),
      ],
    ];
    DatasetBatchOperations::operationProcess((int) $dataset->id(), $context);
    DatasetBatchOperations::finished(TRUE, $context['results']);
    $this->assertEquals([new TranslatableMarkup('Dataset %name was successfully processed.', [
      '%name' => $dataset->label(),
    ]),
    ], \Drupal::messenger()->messagesByType(MessengerInterface::TYPE_STATUS));
  }

  /**
   * Tests finished without processing.
   */
  public function testFinishedWithoutProcessing(): void {
    $dataset = $this->createTestDataset([
      'status' => DatasetInterface::STATUS_PENDING_PROCESSING,
    ]);
    $context = [
      'results' => [
        'dataset_id' => $dataset->id(),
      ],
    ];
    DatasetBatchOperations::finished(TRUE, $context['results']);
    $this->assertEquals([new TranslatableMarkup('Dataset %name was unable to be processed. Please check <a href=":url">the logs</a>.', [
      ':url' => $dataset->toUrl()->toString(),
      '%name' => $dataset->label(),
    ]),
    ], \Drupal::messenger()->messagesByType(MessengerInterface::TYPE_ERROR));
  }

  /**
   * Tests finished with error.
   */
  public function testFinishedWithError(): void {
    $context = [
      'results' => [],
    ];
    DatasetBatchOperations::finished(TRUE, $context['results']);
    $this->assertEquals([new TranslatableMarkup('The validation and processing operation did not complete.')], \Drupal::messenger()->messagesByType(MessengerInterface::TYPE_ERROR));
  }

}
